from dbt.tests.adapter.utils.test_null_compare import (
    BaseMixedNullCompare,
    BaseNullCompare,
)


class TestExasolMixedNullCompare(BaseMixedNullCompare):
    pass


class TestExasolNullCompareBaseNullCompare(BaseNullCompare):
    pass
